@@ -2,20 +2,21 @@
 using System;
 using System.IO;
 using System.Collections;
+using System.Collections.Generic;
 using Mono.Unix;
 
 namespace Tomboy
 {
 	public delegate void NotesChangedHandler (object sender, Note changed);
 
-	public class NoteManager 
+	public class NoteManager
 	{
 		string notes_dir;
 		string backup_dir;
-		ArrayList notes;
-		PluginManager plugin_mgr;
+		List<Note> notes;
+		AddinManager addin_mgr;
 		TrieController trie_controller;
-		
+
 		static string start_note_uri = String.Empty;
 
 		static NoteManager ()
@@ -24,10 +25,10 @@ namespace Tomboy
 			// StartNoteUri property doesn't generate a call to
 			// Preferences.Get () each time it's accessed.
 			start_note_uri =
-				Preferences.Get (Preferences.START_NOTE_URI) as string;
+			        Preferences.Get (Preferences.START_NOTE_URI) as string;
 			Preferences.SettingChanged += OnSettingChanged;
 		}
-		
+
 		static void OnSettingChanged (object sender, GConf.NotifyEventArgs args)
 		{
 			switch (args.Key) {
@@ -37,24 +38,24 @@ namespace Tomboy
 			}
 		}
 
-		public NoteManager (string directory) : 
-			this (directory, Path.Combine (directory, "Backup")) 
+public NoteManager (string directory) :
+		this (directory, Path.Combine (directory, "Backup"))
 		{
 		}
 
-		public NoteManager (string directory, string backup_directory) 
+		public NoteManager (string directory, string backup_directory)
 		{
 			Logger.Log ("NoteManager created with note path \"{0}\".", directory);
 
 			notes_dir = directory;
 			backup_dir = backup_directory;
-			notes = new ArrayList ();
+			notes = new List<Note> ();
 
 			bool first_run = FirstRun ();
 			CreateNotesDir ();
 
-			plugin_mgr = CreatePluginManager ();
 			trie_controller = CreateTrieController ();
+			addin_mgr = CreateAddinManager ();
 
 			if (first_run) {
 				// First run. Create "Start Here" notes.
@@ -66,33 +67,21 @@ namespace Tomboy
 			Tomboy.ExitingEvent += OnExitingEvent;
 		}
 
-		// Create & populate the Plugins dir. For overriding in test
-		// methods.
-		protected virtual PluginManager CreatePluginManager ()
-		{
-			// Allow developers to override the plugins path with an
-			// environment variable.
-			string plugins_dir =
-				Environment.GetEnvironmentVariable ("TOMBOY_PLUGINS_PATH");
-
-			if (plugins_dir == null) {
-				string tomboy_dir = 
-					Path.Combine (Environment.GetEnvironmentVariable ("HOME"), 
-						      ".tomboy");
-				plugins_dir = Path.Combine (tomboy_dir, "Plugins");
-			}
-
-			PluginManager.CreatePluginsDir (plugins_dir);
-
-			return new PluginManager (plugins_dir);
-		}
-
 		// Create the TrieController. For overriding in test methods.
 		protected virtual TrieController CreateTrieController ()
 		{
 			return new TrieController (this);
 		}
 
+		protected virtual AddinManager CreateAddinManager ()
+		{
+			string tomboy_conf_dir =
+			        Path.Combine (Environment.GetEnvironmentVariable ("HOME"),
+			                      ".tomboy");
+
+			return new AddinManager (tomboy_conf_dir);
+		}
+
 		// For overriding in test methods.
 		protected virtual bool DirectoryExists (string directory)
 		{
@@ -124,70 +113,81 @@ namespace Tomboy
 			if (NoteRenamed != null)
 				NoteRenamed (note, old_title);
 		}
-		
+
 		void OnNoteSave (Note note)
 		{
 			if (NoteSaved != null)
 				NoteSaved (note);
 		}
 
-		protected virtual void CreateStartNotes () 
+		protected virtual void CreateStartNotes ()
 		{
 			// FIXME: Delay the creation of the start notes so the panel/tray
 			// icon has enough time to appear so that Tomboy.TrayIconShowing
 			// is valid.  Then, we'll be able to instruct the user where to
 			// find the Tomboy icon.
 			//string icon_str = Tomboy.TrayIconShowing ?
-			//					Catalog.GetString ("System Tray Icon area") :
-			//					Catalog.GetString ("GNOME Panel");
-			string start_note_content = 
-				Catalog.GetString ("<note-content>" +
-						"Start Here\n\n" +
-						"<bold>Welcome to Tomboy!</bold>\n\n" +
-						"Use this \"Start Here\" note to begin organizing " +
-						"your ideas and thoughts.\n\n" +
-						"You can create new notes to hold your ideas by " +
-						"selecting the \"Create New Note\" item from the " +
-						"Tomboy Notes menu in your GNOME Panel. " +
-						"Your note will be saved automatically.\n\n" +
-						"Then organize the notes you create by linking " +
-						"related notes and ideas together!\n\n" +
-						"We've created a note called " +
-						"<link:internal>Using Links in Tomboy</link:internal>.  " +
-						"Notice how each time we type <link:internal>Using " +
-						"Links in Tomboy</link:internal> it automatically " +
-						"gets underlined?  Click on the link to open the note." +
-						"</note-content>");
+			//     Catalog.GetString ("System Tray Icon area") :
+			//     Catalog.GetString ("GNOME Panel");
+			string start_note_content =
+			        Catalog.GetString ("<note-content>" +
+			                           "Start Here\n\n" +
+			                           "<bold>Welcome to Tomboy!</bold>\n\n" +
+			                           "Use this \"Start Here\" note to begin organizing " +
+			                           "your ideas and thoughts.\n\n" +
+			                           "You can create new notes to hold your ideas by " +
+			                           "selecting the \"Create New Note\" item from the " +
+			                           "Tomboy Notes menu in your GNOME Panel. " +
+			                           "Your note will be saved automatically.\n\n" +
+			                           "Then organize the notes you create by linking " +
+			                           "related notes and ideas together!\n\n" +
+			                           "We've created a note called " +
+			                           "<link:internal>Using Links in Tomboy</link:internal>.  " +
+			                           "Notice how each time we type <link:internal>Using " +
+			                           "Links in Tomboy</link:internal> it automatically " +
+			                           "gets underlined?  Click on the link to open the note." +
+			                           "</note-content>");
 
 			string links_note_content =
-				Catalog.GetString ("<note-content>" +
-						"Using Links in Tomboy\n\n" +
-						"Notes in Tomboy can be linked together by " +
-						"highlighting text in the current note and clicking" +
-						" the <bold>Link</bold> button above in the toolbar.  " +
-						"Doing so will create a new note and also underline " +
-						"the note's title in the current note.\n\n" +
-						"Changing the title of a note will update links " +
-						"present in other notes.  This prevents broken links " +
-						"from occurring when a note is renamed.\n\n" +
-						"Also, if you type the name of another note in your " +
-						"current note, it will automatically be linked for you." +
-						"</note-content>");
+			        Catalog.GetString ("<note-content>" +
+			                           "Using Links in Tomboy\n\n" +
+			                           "Notes in Tomboy can be linked together by " +
+			                           "highlighting text in the current note and clicking" +
+			                           " the <bold>Link</bold> button above in the toolbar.  " +
+			                           "Doing so will create a new note and also underline " +
+			                           "the note's title in the current note.\n\n" +
+			                           "Changing the title of a note will update links " +
+			                           "present in other notes.  This prevents broken links " +
+			                           "from occurring when a note is renamed.\n\n" +
+			                           "Also, if you type the name of another note in your " +
+			                           "current note, it will automatically be linked for you." +
+			                           "</note-content>");
+			
+			string tags_note_content = Catalog.GetString ("<note-content>"+
+			                                              "Tagging In Tomboy\n\n"+
+			                                              "Notes can be tagged with the tag bar "+
+			                                              "at the bottom of the note screen. "+
+			                                              "Pressing <bold>Tab</bold> in the tag bar will "+
+			                                              "autocomplete the nearest tag. To confirm the displayed tag "+
+			                                              "press <bold>Enter</bold>."+
+			                                              "</note-content>");
 
 			try {
 				Note start_note = Create (Catalog.GetString ("Start Here"),
-								start_note_content);
+				                          start_note_content);
 				start_note.QueueSave (true);
 				Preferences.Set (Preferences.START_NOTE_URI, start_note.Uri);
 
 				Note links_note = Create (Catalog.GetString ("Using Links in Tomboy"),
-								links_note_content);
+				                          links_note_content);
 				links_note.QueueSave (true);
 				
+				Note tag_note = Create (Catalog.GetString ("Tags in Tomboy"),tags_note_content);
+				tag_note.QueueSave(true);
 				start_note.Window.Show ();
 			} catch (Exception e) {
 				Logger.Warn ("Error creating start notes: {0}\n{1}",
-						e.Message, e.StackTrace);
+				             e.Message, e.StackTrace);
 			}
 		}
 
@@ -205,34 +205,34 @@ namespace Tomboy
 					}
 				} catch (System.Xml.XmlException e) {
 					Logger.Log ("Error parsing note XML, skipping \"{0}\": {1}",
-						    file_path,
-						    e.Message);
+					            file_path,
+					            e.Message);
 				}
 			}
 
-			// Update the trie so plugins can access it, if they want.
+			// Update the trie so addins can access it, if they want.
 			trie_controller.Update ();
-			
-			bool startup_notes_enabled = (bool)
-					Preferences.Get (Preferences.ENABLE_STARTUP_NOTES);
 
-			// Load all the plugins for our notes.
+			bool startup_notes_enabled = (bool)
+			                             Preferences.Get (Preferences.ENABLE_STARTUP_NOTES);
+
+			// Load all the addins for our notes.
 			// Iterating through copy of notes list, because list may be
-			// changed when loading plugins.
-			ArrayList notesCopy = new ArrayList (notes);
+			// changed when loading addins.
+			List<Note> notesCopy = new List<Note> (notes);
 			foreach (Note note in notesCopy) {
-				plugin_mgr.LoadPluginsForNote (note);
-				
+				addin_mgr.LoadAddinsForNote (note);
+
 				// Show all notes that were visible when tomboy was shut down
 				if (note.IsOpenOnStartup) {
 					if (startup_notes_enabled)
 						note.Window.Show ();
-					
+
 					note.IsOpenOnStartup = false;
 					note.QueueSave (false);
 				}
 			}
-			
+
 			// Make sure that a Start Note Uri is set in the preferences.  This
 			// has to be done here for long-time Tomboy users who won't go
 			// through the CreateStartNotes () process.
@@ -246,6 +246,16 @@ namespace Tomboy
 
 		void OnExitingEvent (object sender, EventArgs args)
 		{
+			// Call ApplicationAddin.Shutdown () on all the known ApplicationAddins
+			foreach (ApplicationAddin addin in addin_mgr.GetApplicationAddins ()) {
+				try {
+					addin.Shutdown ();
+				} catch (Exception e) {
+					Logger.Warn ("Error calling {0}.Shutdown (): {1}",
+					             addin.GetType ().ToString (), e.Message);
+				}
+			}
+
 			Logger.Log ("Saving unsaved notes...");
 
 			foreach (Note note in notes) {
@@ -253,26 +263,26 @@ namespace Tomboy
 				// next startup
 				if (note.HasWindow && note.Window.Visible)
 					note.IsOpenOnStartup = true;
-				
+
 				note.Save ();
 			}
 		}
 
-		public void Delete (Note note) 
+		public void Delete (Note note)
 		{
 			if (File.Exists (note.FilePath)) {
 				if (backup_dir != null) {
 					if (!Directory.Exists (backup_dir))
 						Directory.CreateDirectory (backup_dir);
 
-					string backup_path = 
-						Path.Combine (backup_dir, 
-							      Path.GetFileName (note.FilePath));
+					string backup_path =
+					        Path.Combine (backup_dir,
+					                      Path.GetFileName (note.FilePath));
 					if (File.Exists (backup_path))
 						File.Delete (backup_path);
 
 					File.Move (note.FilePath, backup_path);
-				} else 
+				} else
 					File.Delete (note.FilePath);
 			}
 
@@ -287,8 +297,12 @@ namespace Tomboy
 
 		string MakeNewFileName ()
 		{
-			Guid guid = Guid.NewGuid ();
-			return Path.Combine (notes_dir, guid.ToString () + ".note");
+			return MakeNewFileName (Guid.NewGuid ().ToString ());
+		}
+
+		string MakeNewFileName (string guid)
+		{
+			return Path.Combine (notes_dir, guid + ".note");
 		}
 
 		// Create a new note with a generated title
@@ -298,8 +312,8 @@ namespace Tomboy
 			string temp_title;
 
 			while (true) {
-				temp_title = String.Format (Catalog.GetString ("New Note {0}"), 
-							    ++new_num);
+				temp_title = String.Format (Catalog.GetString ("New Note {0}"),
+				                            ++new_num);
 				if (Find (temp_title) == null)
 					break;
 			}
@@ -333,9 +347,24 @@ namespace Tomboy
 			return title;
 		}
 
+		public Note Create (string title)
+		{
+			return CreateNewNote (title, null);
+		}
+
+		public Note Create (string title, string xml_content)
+		{
+			return CreateNewNote (title, xml_content, null);
+		}
+
+		public Note CreateWithGuid (string title, string guid)
+		{
+			return CreateNewNote (title, guid);
+		}
+
 		// Create a new note with the specified title, and a simple
 		// "Describe..." body which will be selected for easy overwrite.
-		public Note Create (string title) 
+		private Note CreateNewNote (string title, string guid)
 		{
 			string body = null;
 
@@ -347,15 +376,15 @@ namespace Tomboy
 				body = Catalog.GetString ("Describe your new note here.");
 
 			string header = title + "\n\n";
-			string content = 
-				String.Format ("<note-content>{0}{1}</note-content>",
-					       XmlEncoder.Encode (header),
-					       XmlEncoder.Encode (body));
+			string content =
+			        String.Format ("<note-content>{0}{1}</note-content>",
+			                       XmlEncoder.Encode (header),
+			                       XmlEncoder.Encode (body));
 
-			Note new_note = Create (title, content);
+			Note new_note = CreateNewNote (title, content, guid);
 
-			// Select the inital "Describe..." text so typing will
-			// immediately overwrite...
+			// Select the inital
+			// "Describe..." text so typing will overwrite the body text,
 			NoteBuffer buffer = new_note.Buffer;
 			Gtk.TextIter iter = buffer.GetIterAtOffset (header.Length);
 			buffer.MoveMark (buffer.SelectionBound, iter);
@@ -365,15 +394,19 @@ namespace Tomboy
 		}
 
 		// Create a new note with the specified Xml content
-		public Note Create (string title, string xml_content)
+		private Note CreateNewNote (string title, string xml_content, string guid)
 		{
 			if (title == null || title == string.Empty)
 				throw new Exception ("Invalid title");
 
 			if (Find (title) != null)
-				throw new Exception ("A note with this title already exists");
+				throw new Exception ("A note with this title already exists: " + title);
 
-			string filename = MakeNewFileName ();
+			string filename;
+			if (guid != null)
+				filename = MakeNewFileName (guid);
+			else
+				filename = MakeNewFileName ();
 
 			Note new_note = Note.CreateNewNote (title, filename, this);
 			new_note.XmlContent = xml_content;
@@ -382,8 +415,8 @@ namespace Tomboy
 
 			notes.Add (new_note);
 
-			// Load all the plugins for the new note
-			plugin_mgr.LoadPluginsForNote (new_note);
+			// Load all the addins for the new note
+			addin_mgr.LoadAddinsForNote (new_note);
 
 			if (NoteAdded != null)
 				NoteAdded (this, new_note);
@@ -391,7 +424,7 @@ namespace Tomboy
 			return new_note;
 		}
 
-		public Note Find (string linked_title) 
+		public Note Find (string linked_title)
 		{
 			foreach (Note note in notes) {
 				if (note.Title.ToLower () == linked_title.ToLower ())
@@ -409,41 +442,33 @@ namespace Tomboy
 			return null;
 		}
 
-		class CompareDates : IComparer
+		class CompareDates : IComparer<Note>
 		{
-			public int Compare (object a, object b)
+			public int Compare (Note a, Note b)
 			{
-				Note note_a = a as Note;
-				Note note_b = b as Note;
-
 				// Sort in reverse chrono order...
-				if (note_a == null || note_b == null)
+				if (a == null || b == null)
 					return -1;
 				else
-					return DateTime.Compare (note_b.ChangeDate, 
-								 note_a.ChangeDate);
+					return DateTime.Compare (b.ChangeDate,
+					                         a.ChangeDate);
 			}
 		}
-		
+
 		public static string StartNoteUri
 		{
-			get { return start_note_uri; }
+			get {
+				return start_note_uri;
+			}
 		}
 
-		public ArrayList Notes 
+		public List<Note> Notes
 		{
 			get {
 				// FIXME: Only sort on change by listening to
 				//        Note.Saved or Note.Buffer.Changed
 				notes.Sort (new CompareDates ());
-				return notes; 
-			}
-		}
-
-		public PluginManager PluginManager
-		{
-			get {
-				return plugin_mgr;
+				return notes;
 			}
 		}
 
@@ -454,6 +479,20 @@ namespace Tomboy
 			}
 		}
 
+		public AddinManager AddinManager
+		{
+			get {
+				return addin_mgr;
+			}
+		}
+
+		public string NoteDirectoryPath
+		{
+			get {
+				return notes_dir;
+			}
+		}
+
 		public event NotesChangedHandler NoteDeleted;
 		public event NotesChangedHandler NoteAdded;
 		public event NoteRenameHandler NoteRenamed;
@@ -499,9 +538,11 @@ namespace Tomboy
 			}
 		}
 
-		public TrieTree TitleTrie 
+		public TrieTree TitleTrie
 		{
-			get { return title_trie; }
+			get {
+				return title_trie;
+			}
 		}
 	}
 }
